import GanttChartOptions from './gantt-chart-options.js';

import CanvasContainer from '../references/canvas-container.js';
import BackgroundCanvas from './background-canvas.js';
import ForegroundCanvas from './foreground-canvas.js';
import FlourishCanvas from './flourish-canvas.js';

import './project.scss';


class GanttChart {
  Options = null;
  CanvasContainer = null;

  #debug = false;
  Tasks = [];
  StartDate = null;
  Duration = 30;
  StartOfWeek = 1;


  constructor(el, options) {
    const a = this;

    a.Options = Object.assign(new GanttChartOptions(), options);
    a.CanvasContainer = new CanvasContainer(el);

    a.#initialiseComponents();
  }

  #initialiseComponents() {
    const a = this;

    // Add background canvas layer
    const layer1 = a.CanvasContainer.AddLayer();

    const backCanvasLayer = new BackgroundCanvas(layer1);
    backCanvasLayer.Options = a.Options;

    a.CanvasContainer.Layer.push(backCanvasLayer);

    // Add foreground canvas layer
    const layer2 = a.CanvasContainer.AddLayer();

    const foreCanvasLayer = new ForegroundCanvas(layer2);
    foreCanvasLayer.Options = a.Options;

    a.CanvasContainer.Layer.push(foreCanvasLayer);

    // Add flourish canvas layer
    const layer3 = a.CanvasContainer.AddLayer();

    const flourishCanvasLayer = new FlourishCanvas(layer3);
    flourishCanvasLayer.Options = a.Options;

    a.CanvasContainer.Layer.push(flourishCanvasLayer);

    // Invalidate every canvas
    a.CanvasContainer.Invalidate();

    Document.removeClass(a.CanvasContainer.FlowContainer, "border");
    Document.addClass(a.CanvasContainer.FlowContainer, "gantt-chart");
  }


  get HeaderHeight() {
    const a = this;

    return a.Options.HeaderRow.Height[0] + a.Options.HeaderRow.Height[1];
  }

  get Debug() {
    return this.#debug;
  }

  set Debug(value) {
    const a = this;

    a.#debug = value;

    a.CanvasContainer.Layer.forEach(e => {
      if (typeof(e.Debug) == "undefined") {
        return;
      }

      e.Debug = value;
    });

  }


  Clear() {
    const a = this;

    a.CanvasContainer.Clear();

    a.StartDate = new Date();
    a.Duration = 30;
    a.StartOfWeek = 1;    // 1 = Monday
    a.Tasks = [];

    // Invalidate canvas (background, foreground)
    a.CanvasContainer.Layer[0].Load(new Date(), a.Duration, a.StartOfWeek, a.Tasks.length);
    a.CanvasContainer.Layer[1].Load(new Date(), a.Tasks);
    a.CanvasContainer.Layer[2].Load();

    a.Invalidate();
  }

  Invalidate() {
    const a = this;

    a.CanvasContainer.Clear();

    const width = ((a.Duration + 2) * a.Options.DayWidth);
    const height = (Math.max(a.Tasks.length, a.Options.MinimumRowCount) * a.Options.Row.Height) + a.HeaderHeight;

    a.CanvasContainer.Size = {
      W: width,
      H: height
    };

    a.CanvasContainer.Invalidate();
  }

  Load(project) {
    const a = this;

    a.CanvasContainer.Clear();

    if (project == null) {
      return;
    }

    a.StartDate = new Date(project.StartDate);
    a.Duration = project.Duration;
    a.StartOfWeek = project.Project.StartOfWeek;
    a.Tasks = project.ExportTasks();

    // Invalidate canvas (background, foreground)
    a.CanvasContainer.Layer[0].Load(new Date(project.StartDate), project.Duration, project.Project.StartOfWeek, a.Tasks.length);
    a.CanvasContainer.Layer[1].Load(new Date(project.StartDate), a.Tasks);
    a.CanvasContainer.Layer[2].Load();

    a.Invalidate();
  }

}


export default GanttChart;